// src/backend/cpu_policy_manager.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef CPU_POLICY_MANAGER_HH
#define CPU_POLICY_MANAGER_HH 1

namespace sgpem
{
  class CPUPolicyManager;
}

#include "config.h"

#include <sgpemv2/cpu_policy.hh>

#include <vector>


namespace sgpem
{
  /**
    \brief CPUPolicyManager is the Abstract Factory for 
    \ref CPUPolicy objects.
  */
  class SG_DLLEXPORT CPUPolicyManager
  {
  public:
    typedef CPUPolicy Policy;
    typedef std::vector<Policy*> Policies;
    
    /** \brief CPUPolicyManager constructor
     *
    * Registers the \b this pointer to the CPUPoliciesGatekeeper, so it can be accessed
    * when needed. This is done so that concrete subclasses can be defined
    * even if they are found in external dynamic modules not known at compile time.
    */
    CPUPolicyManager();

    virtual ~CPUPolicyManager() = 0;

    virtual const Policies& get_avail_policies() = 0;

  protected:
    /** 
     * \brief Collects available policies
     * 
     * Overriders should implement this method such that it
     * collects available policies (an operation which may need to be done
     * at run-time in case user-defined policies are to be supported) 
     * and puts them inside _policies. The application then assumes this 
     * method will be called at some time during the initialization of 
     * the manager (usually inside the constructor).
     */
    virtual void collect_policies() = 0;

    std::vector<CPUPolicy*> _policies;
  };

} //~ namespace sgpem

#endif

